﻿using Exam.Infrastructure.Utilities;
using System.Reflection;
using System.Text.RegularExpressions;

namespace Exam.Infrastructure.Web.Utilities
{
    public class AssemblyUtility : BaseAssemblyUtility
    {


        private static string[] FinderFilter
        {
            get
            {
                return new string[] { API, CLIENT, EXTENSIONS };
            }
        }
        private static string[] ModuleFilter
        {
            get
            {
                return new string[] { @"^Exam\.(.*\.)?Application", @"^Exam\.(.*\.)?Share" };
            }
        }

        public static List<Assembly> GetAssembly(string path)
        {
            //dynamic type = (new Program()).GetType();
            string currentDirectory = Path.GetDirectoryName(path);
            var files = Directory.GetFiles(currentDirectory, ALL);
            var assemblys = new List<Assembly>();

            foreach (var file in files)
            {
                assemblys.Add(Assembly.LoadFrom(file));
            }

            return assemblys;
        }

        public static List<Assembly> CreateModules(List<Assembly> assemblies)
        {
            List<Assembly> modules = new List<Assembly>();
            foreach (var filter in ModuleFilter)
            {
                CreateModulesByFilter(assemblies, filter);
            }
            return modules;
        }

        public static List<Assembly> CreateModulesByFilter(List<Assembly> assemblies, string filter)
        {
            List<Assembly> modules = new List<Assembly>();
            modules.AddRange(
                assemblies.Where(item => Regex.IsMatch(Path.GetFileNameWithoutExtension(item.CodeBase), filter)));
            return modules;
        }

        public static Dictionary<string, List<Assembly>> CreateFinders(List<Assembly> assemblies)
        {
            Dictionary<string, List<Assembly>> finders = new Dictionary<string, List<Assembly>>();
            foreach (var filter in FinderFilter)
            {
                if (!finders.ContainsKey(filter))
                {
                    finders[filter] = new List<Assembly>();
                }
                finders[filter]
                    .AddRange(assemblies.Where(item => Regex.IsMatch(Path.GetFileNameWithoutExtension(item.CodeBase), filter)));
            }
            return finders;
        }



        public static Tuple<Dictionary<string, List<Assembly>>, Assembly[]> ScannerAssemblies(string path)
        {
            var assemblies = GetAssembly(path);

            var finders = CreateFinders(assemblies);

            var modules = CreateModules(assemblies);

            return new Tuple<Dictionary<string, List<Assembly>>, Assembly[]>(finders, modules.ToArray());
        }

        public static List<Type> GetTypeToRegister<TEntity>(List<Assembly> assemblies, string filter)
        {
            var types = new List<Type>();

            var currentDomainAssemblies = CreateModulesByFilter(assemblies, filter);


            foreach (var currentAssembly in currentDomainAssemblies)
            {
                types.AddRange(currentAssembly.GetTypes().Where(q => q.GetInterface(typeof(TEntity).FullName) != null));

            }

            return types;
        }

        public static List<TEntity> CreateInstance<TEntity>(List<Assembly> assemblies, string filter)
        {
            var instatnces = new List<TEntity>();

            var currentDomainAssemblies = CreateModulesByFilter(assemblies, filter);

            foreach (var currentAssembly in currentDomainAssemblies)
            {
                var types = currentAssembly.GetTypes().Where(q => q.GetInterface(typeof(TEntity).FullName) != null);

                foreach (var type in types)
                {
                    instatnces.Add((TEntity)currentAssembly.CreateInstance(type.FullName));
                }

            }

            return instatnces;
        }
    }
}
