﻿using Exam.Application.Interface.Questions;
using Exam.Infrastructure.Data.Entity;
using Exam.Infrastructure.Data.Extensions;
using Exam.Share.ViewResponses.Question;
using Hotline.Application.Exam.Constants.ApiRoutes;
using Hotline.Share.Dtos.Questions;
using Hotline.Share.Requests.Question;
using Microsoft.AspNetCore.Mvc;
using XF.Domain.Authentications;

namespace Hotline.Api.Controllers.Exam
{
    public class QuestionController : BaseController
    {
        private readonly IQuestionService _questionService;
        private readonly ISessionContext _sessionContext;
        public QuestionController(IQuestionService questionService,ISessionContext sessionContext)
        {
            _questionService = questionService;
            _sessionContext = sessionContext;
        }

        /// <summary>
        /// 新增题库
        /// </summary>
        /// <param name="questionDto"></param>
        /// <returns></returns>
        [HttpPost(QuestionApiRoute.Add)]
        public async Task Add([FromBody] QuestionDto questionDto)
        {
            questionDto.InitRequest(_sessionContext);
            await _questionService.AddAsync(questionDto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 修改题库
        /// </summary>
        /// <param name="questionDto"></param>
        /// <returns></returns>
        [HttpPut(QuestionApiRoute.Update)]
        public async Task Update([FromBody] QuestionDto questionDto)
        {
            questionDto.InitRequest(_sessionContext);
            await _questionService.UpdateAsync(questionDto, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 删除题库
        /// </summary>
        /// <param name="entityQueryRequest"></param>
        /// <returns></returns>
        [HttpDelete(QuestionApiRoute.Delete)]
        public async Task Delete([FromBody] EntityQueryRequest entityQueryRequest)
        {
            await _questionService.DeleteAsync(entityQueryRequest, HttpContext.RequestAborted);
        }

        /// <summary>
        /// 获取题库分页列表
        /// </summary>
        /// <param name="questionPagedRequest"></param>
        /// <returns></returns>
        [HttpPost(QuestionApiRoute.GetPagedList)]
        public async Task<QuestionPageViewResponse> GetPagedList([FromBody] QuestionPagedRequest questionPagedRequest)
        {
            var questionPageViewResponse = await _questionService.GetPagedListAsync(questionPagedRequest);

            return questionPageViewResponse as QuestionPageViewResponse;
        }

        /// <summary>
        /// 获取题库
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet(QuestionApiRoute.Get)]
        public async Task<QuestionDto> Get(string id)
        {
            var questionDto = await _questionService.GetAsync(new EntityQueryRequest
            {
                Id = id
            });

            return questionDto;
        }
    }
}
