using Exam.Application.QueryExtensions.Sourcewares;
using Exam.Infrastructure.Data.Entity;
using Exam.Infrastructure.Extensions;
using Exam.Insfrastructure.Service.Service;
using Exam.Share.ViewResponses.Sourceware;
using Hotline.Application.Exam.Constants.Messages;
using Hotline.Application.Exam.Extensions;
using Hotline.Exams.Sourcewares;
using Hotline.Repository.SqlSugar;
using Hotline.Repository.SqlSugar.Exam.Interfaces.Sourcewares;
using Hotline.Share.Dtos.Sourcewares;
using Hotline.Share.Requests.Sourceware;
using Mapster;
using System.ComponentModel;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Exam.Application.Service.Sourcewares
{
    /// <summary>
    /// 课件类型服务
    /// </summary>
    [Description("课件类型服务")]
    public class SourcewareCategoryService : ApiService<SourcewareCategory, SourcewareCategoryDto,HotlineDbContext>,ISourcewareCategoryService, IScopeDependency
    {
        private readonly ISourcewareCategoryRepository _repository;
        public SourcewareCategoryService(ISourcewareCategoryRepository repository) : base(repository)
        {
            _repository = repository;
        }

        public async Task<SourcewareCategoryDto> GetAsync(EntityQueryRequest entityQueryRequest)
        {
            var entity = await _repository.GetAsync(entityQueryRequest.Id);

            return entity.Adapt<SourcewareCategoryDto>();
        }

        public async Task<(int, List<SourcewareCategoryViewResponse>)> GetListAsync(SourcewareCategoryRequest queryRequest)
        {
            var expression = queryRequest.GetExpression();
            var query = _repository.Queryable().Where(expression);

            var result = await query.Select(m => new SourcewareCategoryViewResponse { 
                				Name = m.Name,
								ParentId = m.ParentId,
								Id = m.Id,
				            }).ToListAsync();

            var total = await query.CountAsync();

            return (total,result);
        }

        public async Task<PageViewResponse<SourcewareCategoryViewResponse>> GetPagedListAsync(SourcewareCategoryRequest queryRequest)
        {
            throw new NotImplementedException();
        }

        /// <summary>
        /// 获取树状
        /// </summary>
        /// <param name="sourcewareCategoryDto"></param>
        /// <returns></returns>
        /// <exception cref="NotImplementedException"></exception>
        public async Task<List<SourcewareCategoryViewResponse>> GetTreeAsync(SourcewareCategoryRequest sourcewareCategoryRequest)
        {
            var expression = sourcewareCategoryRequest.GetExpression();

            var sourcewareCategorys = await _repository.GetTreeAsync(expression);

            var sourcewareCategoryViewResponses = sourcewareCategorys.Select(m => new SourcewareCategoryViewResponse
            {
                Id = m.Id,
                Name = m.Name,
                ParentId = m.ParentId,
                SortIndex = m.SortIndex,
                LastModificationTime = m.LastModificationTime,
                Children = m.Children.Adapt<List<SourcewareCategoryViewResponse>>()
            }).ToList();


            return sourcewareCategoryViewResponses;
        }

        /// <summary>
        /// 删除课件分类
        /// </summary>
        /// <param name="entityQueryRequest"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public override Task DeleteAsync(EntityQueryRequest entityQueryRequest, CancellationToken cancellationToken)
        {
            //TODO 如果分类拥有下级分类怎么不能删除

            if (_repository.CheckHasChild(entityQueryRequest))
            {
                throw new UserFriendlyException(BusinessErrorMessage.ServiceError, string.Format(BusinessErrorMessage.HasChild,typeof(SourcewareCategory).GetDescription()));
            }

            return base.DeleteAsync(entityQueryRequest, cancellationToken);
        }
    }
}


